package convertquery

import "fmt"

// createConstraintStatements generates the appropriate amount of constraint lines calling createConstraingBoolExpression
func createConstraintStatements(constraints *[]constraintStruct, name string, isRelation bool) *string {
	s := ""
	if len(*constraints) == 0 {
		return &s
	}

	newLineStatement := "\tFILTER"

	for _, v := range *constraints {
		s += fmt.Sprintf("%v %v \n", newLineStatement, *createConstraintBoolExpression(&v, name, isRelation))
		newLineStatement = "\tAND"
	}

	return &s
}

// createConstraintBoolExpression generates a bool expression, e.g. {name}.city == "New York".
// when isRelation is true, {name}.edges[*] ALL == "New York"
func createConstraintBoolExpression(constraint *constraintStruct, name string, isRelation bool) *string {
	var (
		match string
		value string
		line  string
	)

	//Wicked switches letsgo
	switch constraint.DataType {
	case "text":
		value = fmt.Sprintf("\"%s\"", constraint.Value)
		switch constraint.MatchType {
		case "contains":
			match = "IN"
		case "startswith":
			match = "LIKE"
			value = fmt.Sprintf("\"%s%%\"", constraint.Value)
		case "endswith":
			match = "LIKE"
			value = fmt.Sprintf("\"_%s\"", constraint.Value)
		default: //exact
			match = "=="
		}
	case "number":
		value = constraint.Value
		switch constraint.MatchType {
		case "GT":
			match = ">"
		case "LT":
			match = "<"
		case "GET":
			match = ">="
		case "LET":
			match = "<="
		default: //EQ
			match = "=="
		}
	default: /*bool*/
		value = constraint.Value
		switch constraint.MatchType {
		case "NEQ":
			match = "!="
		default: //EQ
			match = "=="
		}
	}

	if isRelation {
		line = fmt.Sprintf("%s.edges[*].%s ALL %s %s", name, constraint.Attribute, match, value)
	} else {
		line = fmt.Sprintf("%s.%s %s %s", name, constraint.Attribute, match, value)
	}
	return &line
}
